@@ -17,6 +17,15 @@ sub new
     return $self;
 }
 
+sub parse_request
+{
+    my $self = shift;
+#   my $content = $_[0]->decoded_content;
+#   # JSONP to pure JSON
+#   $content =~ s/[a-zA-Z_\$][a-zA-Z0-9_\$]*\s*\((.+)\)\s*;?\s*$/$1/;
+    $self->{json}->encode( $_[0] );
+}
+
 sub parse_response
 {
     my $self = shift;
